@@ -39,10 +39,12 @@ mlflow_set_tracking_uri <- function(uri) {
 
 #' Get Remote Tracking URI
 #'
+#' Gets the remote tracking URI.
+#'
 #' @export
 mlflow_get_tracking_uri <- function() {
   .globals$tracking_uri %||% {
     env_uri <- Sys.getenv("MLFLOW_TRACKING_URI")
-    if (nchar(env_uri)) env_uri else fs::path_abs("mlruns")
+    if (nchar(env_uri)) env_uri else paste("file://", fs::path_abs("mlruns"), sep = "")
   }
 }
